
;***************************************************************
;*		Anschlussplan Display zum AVR		       *
;*-------------------------------------------------------------*
;*    Pin Belegung Display	: Pin Belegung AT90S8515       *
;*-------------------------------------------------------------*
;* 1.  GND                      :	GND		       *
;* 2.  +5V			:	+5V		       *
;* 3.  Vo Contrast		:  ber 10k Poti auf +5V       *
;* 4.  RS Daten oder Code Input	: 27.	PC6		       *
;* 5.  R/W 			: 16.	PD6		       *
;* 6.  Enable			: 23.	PC2		       *
;* 7.  DB0			: 39. 	PA0		       *
;* 8.  DB1			: 38.	PA1		       *
;* 9.  DB2			: 37.	PA2		       *
;* 10. DB3			: 36.	PA3		       *
;* 11. DB4			: 35.	PA4		       *
;* 12. DB5			: 34.	PA5		       *
;* 13. DB6			: 33.	PA6		       *
;* 14. DB7			: 32.	PA7		       *
;* 15. Anode  LED		:	+5V		       *
;* 16. Katode LED		:	GND		       *
;***************************************************************

.nolist
.list									

;**** Definierung der Steuerleitungen ****

.equ	E	=	PC7				; Enable Leitung
.equ	RW	=	PC6					; Read / Write Leitung
.equ	RS	=	PC1					; Befehlsleitung
.equ	Busy	=	PB7					; Busy Leitung 

; **** Definierung der Befehle ****

.equ	lcdon	=	0b00001111				; LCD einschalten
.equ	lcdmode	=	0b00111000				; LCD Mode 8 Bit, 2 Zeilen
.equ	lcdclear=	0b00000001				; LCD lschen
.equ	lcdentry=	0b00000110				; bei eingabe eines Zeichen den Cursor um 1 erhhen
.equ	zeile2	=	0b11000000				; Startadresse der zweiten Display Zeile	
.equ	zeile4	=	0b11010100
.equ	zeile1	=	0b10000000
.equ	zeile3	=	0b10010100

; **** Definierung der Arbeitsregister *****

;.def	temp1	=	r16					; Arbeitsregister 1
;.def	temp2_LCD	=	r2					; Arbeitsregister 2
;.def	temp3	=	r18					; Arbeitsregister 3
.def	temp4_LCD	=	r3					; Arbeitsregister 4
.def	tim1	=	r4					; Timerregister 1
.def	tim2	=	r5					; Timerregister 2

; **** Hauptprogramm ****


initlcd:
						

	ldi	LCD_out,0b11111111				;  
	out	ddrb,LCD_out					; Port B auf Ausgang setzten
	out	ddrc,LCD_out					; Port C auf Ausgang setzten

	

; **** Initialisierung des LCD Displays ****



	rcall 	wait15ms					; 15ms auf Display warten

	ldi	LCD_out,lcdmode					; LCD-Mode setzten
	rcall	sendebefehl					; Befehl senden
	rcall	wait15ms					; 15ms warten
	ldi	LCD_out,lcdmode					; LCD-Mode setzten
	rcall	sendebefehl					; Befehl senden
	rcall	wait5ms						; 5ms warten
	ldi	LCD_out,lcdmode					; LCD-Mode setzten
	rcall	sendebefehl					; Befehl senden
	rcall	wait5ms						; 5ms warten
	ldi	LCD_out,lcdon					; LCD einschalten
	rcall	sendebefehl					; Befehl senden
	rcall	wait150us					; 150us warten
	ldi	LCD_out,lcdclear					; LCD lschen
	rcall	sendebefehl					; Befehl senden
	rcall	wait150us					; 150us warten
	ldi	LCD_out,lcdentry					; Entrymode setzten	
	rcall	sendebefehl					; Befehl senden
	rcall	wait5ms						; 5ms warten
	rcall	boot						; gibt "boot" auf LCD aus
	rcall	wait15ms						
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	rcall	ready						; gibt "ready" auf LCD aus
	rcall	wait15ms						
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	rcall	wait15ms
	ldi	LCD_out,lcdclear					; LCD lschen
	rcall	sendebefehl


	ret
loop:	
	rjmp	loop
	
	
; **** Senden von Befehlen und Daten ans Display ****

sendebefehl:
	
	rcall	busyflag
	cbi	portc,RS					; lsche BIT RS	
	cbi	portc,RW					; lsche BIT RW
	rcall	ausgabe						; Befehl ausgeben
	ret

write_LCD_data:

	rcall	busyflag					; Busyflag abfragen
	sbi	portc,RS					; setzten BIT RS
	cbi	portc,RW					; lsche BIT RW
	rcall	ausgabe						; Daten ausgeben
	ret

ausgabe:

	out	portb,LCD_out					; Daten aus Register ausgeben
	sbi	portc,E						; setzten BIT E
	nop							; kein Operation (warten)
	nop
	nop
	cbi	portc,E						; lsche BIT E
	ret
	
busyflag:
	
	clr	temp						; lsche Register temp2_LCD
	out	ddrb,temp					; Port A als Eingnge setzten
	cbi	portc,RS					; lsche BIT RS
	sbi	portc,RW					; setzte BIT RW
wb1:
	sbi	portc,E						; setzten BIT E
	nop							; keine Operation (warten)
	nop
	in	temp,pinb					; lese Port A in Register temp2_LCD ein
	cbi	portc,E						; lsche BIT E
	nop			
	sbrc	temp,Busy					; berspringe nchsten Befehl falls Busy flag 
	rjmp	wb1						; gelscht ist
	cbi	portc,RW					; lsche BIT RW
	;mov temp, temp2_LCD
	ser	temp						; setzte alle BITs im Register temp2_LCD
	out	ddrb,temp					; setzte Port A als Ausgang
	ret
	nop	
	

 
; **** Zeitschleifen ****

wait50us:
	
	ldi	temp,65
	mov tim1, temp						; 50us warten bei 4MHz Tacktfrequenz
	wait51:	
	mov temp, tim1						; 50us / 750ns = 65
	dec	temp
	mov tim1, temp						; 250ns fr DEC Befehl und 500ns fr BRNE 
	brne 	wait51						; Befehl = 750ns
	ret							; 750ns * 65 Durchlufe = 50us 
 
wait150us:

	rcall wait50us
	rcall wait50us
	rcall wait50us
	ret

wait5ms:

	ldi	temp,100
	mov tim2, temp
	wait5:
	rcall	wait50us
	mov temp, tim2
	dec	temp
	mov tim2, temp
	brne	wait5
	ret

wait15ms:

	rcall	wait5ms
	rcall	wait5ms
	rcall	wait5ms
	ret

; **** Definierung der Texte mit Position ****

boot:

	mov temp, temp4_LCD
	ldi	temp,12
	mov temp4_LCD, temp					; lade 15 in Register temp4
	ldi	zl,low(text1*2)				; Zeige auf Z-Pointer Lowbyte von Tabelle1
	ldi	zh,high(text1*2)				; Zeige auf Z-Pointer Highbyte von Tabelle1
loop_msg1:
	lpm							; lade R0 mit Wert auf der der Z-Pointer zeigt  
	mov	LCD_out,r0					; kopiere R0 in LCD_out
	rcall	write_LCD_data					; Daten senden
	inc	zl						; Incrementiere Z-Pointer LOW
	brcc	no_carry1					; Springe falls Carryflag gelscht
	inc	zh						; Incrementiere Z-Pointer High
no_carry1:
	dec	temp4_LCD						; Decrementiere temp4
	brne	loop_msg1					; vergleiche auf Null, wenn nich springe
	ret

ready:

	ldi	LCD_out,zeile2				; setzte Cursor in zweite Zeile (Anfang)
	rcall	sendebefehl
	mov temp, temp4_LCD
	ldi	temp,6
	mov temp4_LCD, temp					; lade 15 in Register temp4
	ldi	zl,low(text2*2)				; Zeige auf Z-Pointer Lowbyte von Tabelle1
	ldi	zh,high(text2*2)				; Zeige auf Z-Pointer Highbyte von Tabelle1
loop_msg2:
	lpm							; lade R0 mit Wert auf der der Z-Pointer zeigt  
	mov	LCD_out,r0					; kopiere R0 in LCD_out
	rcall	write_LCD_data					; Daten senden
	inc	zl						; Incrementiere Z-Pointer LOW
	brcc	no_carry2					; Springe falls Carryflag gelscht
	inc	zh						; Incrementiere Z-Pointer High
no_carry2:
	dec	temp4_LCD						; Decrementiere temp4
	brne	loop_msg2					; vergleiche auf Null, wenn nich springe
	ret


;** Texte die ausgegeben werden **

text1:

.db " booting... "

text2:

.db " ready"

text3:

.db "1.Konstant"

text4: 

.db "2.Spg erhhen/senken"

text5:

.db "3.Spgzu bestimmter t"

text6:

.db "Auswahl:  "

text7:

.db "Spannung eingeben:  "

text8:

.db "Anfangsspg eingeben:"

text9:

.db "dt (in min):  "

text10:

.db "dV (1/10 Volt): "

text11:

.db "Wieviele Spannungen:"

text12:

.db "Spg eingeben: "

text13:

.db "Dauer (in min): "

text14:

.db "Anfangsspannung:"

text15:

.db "Endspannung:"

text16:

.db "Zeit bis Endspg:"

text17:

.db "PROBLEM!!!"

text18:

.db "Fertig :-)"

text19:

.db "2.Spg erhohen/senken"

min:

.db "min "

restzeit:

.db "Restzeit (in min):"

spannung:

.db "Aktuelle Spannung:"
